<?php
class Xoogu_GeoText_Admin extends Xoogu_GeoText
{
	public function __construct()
	{
        parent::__construct();
		//register the activation and deactivation methods
        $mainPluginFile=dirname(__FILE__).'/xoogu-geotext.php';
        register_activation_hook( $mainPluginFile, array($this, 'activate'));
        //register_deactivation_hook( $mainPluginFile, array($this, 'deactivate'));
        register_uninstall_hook( $mainPluginFile, array('Xoogu_GeoText_Admin', 'uninstall'));
        //add admin menu
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('network_admin_menu', array($this, 'add_admin_menu'));
        load_plugin_textdomain('xoogu-geotext', false, dirname(plugin_basename(__FILE__)).'/lang/');
	}
	
	/**
	 *Loads the MCE / Quicktags related class if we're on the edit screen
	 *hook onto 'current_screen' action
	 *@param WP_Screen $current_screen The current screen
	 */
	public function load_MCE_if_edit_screen($current_screen)
	{
		global $Xoogu_GeoText_Admin_MCE;
		if($current_screen->base == 'post'){
			require_once(dirname(__FILE__).'/Xoogu_GeoText_Admin_MCE.php');
			$Xoogu_GeoText_Admin_MCE = new Xoogu_GeoText_Admin_MCE();
		}
	}
    
    
    /**
     *Adds the plugin settings page to the Settings menu
     */
    public function add_admin_menu()
	{
		//Add a new menu item under Settings
        $pageTitle=sprintf(__('%s GeoText','xoogu-geotext'), 'Xoogu');
		if (is_plugin_active_for_network('xoogu-geotext/xoogu-geotext.php')) {
			$hook_suffix = add_submenu_page( 'settings.php', $pageTitle, $pageTitle, 'manage_options', 'xoogu_geotext', array($this, 'display_page') );
			//add settings page link in plugins list
			add_filter('network_admin_plugin_action_links_'.'xoogu-geotext/xoogu-geotext.php', array($this, 'plugin_action_links_network'));
		} else {
			$hook_suffix = add_options_page( $pageTitle, $pageTitle, 'manage_options', 'xoogu_geotext', array($this, 'display_page') );
			//add settings page link in plugins list
			add_filter('plugin_action_links_'.'xoogu-geotext/xoogu-geotext.php', array($this, 'plugin_action_links'));
		}
		add_action( 'admin_print_styles-'.$hook_suffix, array($this, 'adminPageLoad'));
    }
	
	public function adminPageLoad()
	{
		wp_enqueue_style('xoogu_geotext_admin', plugins_url( 'admin.css' , __FILE__ ));
		wp_enqueue_script('xoogu_geotext_admin', plugins_url( 'admin.js' , __FILE__ ), array('jquery'), self::$sVersion, true );
		wp_localize_script( 'xoogu_geotext_admin', 'xgtL10n', array(
			'errGroupingsNoName' => __('There is a grouping with countries selected for it, but no name has been chosen for the grouping','xoogu-geotext'),
			'errGroupingsNoValue' => __('Grouping %s does not have any countries listed to include in it','xoogu-geotext'),
			'errCountryCodeNotValid' => __('Grouping %s contains the country code %s, but this does not appear to be a valid country code','xoogu-geotext'),
			'errGroupingsNoValidValues' => __('Grouping %s does not have any valid countries listed to include in it','xoogu-geotext')
		) );
		
	}
    
    
    /**
     *Display the settings page
     *See also the duplicate in Xoogu_GeoText_Admin_MCE::display_mce_form
     */
    public function display_page()
	{
        if (!current_user_can('manage_options')) {
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="https://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','xoogu-geotext') );
        }
		
		
        //if the form was submitted, process the POST
        if (isset($_POST['submit'])) {
			$errors = $this->validate_settings($_POST);
			if( empty($errors)) {
				?><div class="updated"><p><?php _e('Settings saved successfully.','xoogu-geotext')?></p></div><?php
			}
            //set the form values to those that were submitted
            $formValues = $_POST['options'];
			$formValues['groupings'] = $_POST['groupings'];
        }
        else{
            //otherwise set the form values to the saved options
            $formValues = $this->_aOptions;
			$formValues['groupings'] = $this->getGroupingsCountryLists();
			$formValues['groupings'][] = array('grouping' => '', 'countrylist' => '');
        }
        ?>
        <h1><?php printf( __('%s GeoText Plugin Settings','xoogu-geotext'), 'Xoogu');?></h1>
        <form action="<?php echo (is_plugin_active_for_network('xoogu-geotext/xoogu-geotext.php') && is_network_admin()) ? network_admin_url('settings.php?page=xoogu_geotext') : get_admin_url(null, 'options-general.php?page=xoogu_geotext');?>" method="post" enctype="multipart/form-data" autocomplete="off" id="xoogu_geotext_options_form">
            <?php
            //if there were any errors when we validated the POSTed data, print them out
            if (!empty($errors)) {
                echo '<div class="error">';
				$this->printErrors($errors);
				echo '</div>';
            }?>
            <dl>
                <dt><?php _e('Detect user\'s country from their IP address','xoogu-geotext');?></dt>
                <dd>
                    <div class="desc">
                        <p><?php _e('The plugin supports a number of different Geo IP lookup methods, and you can choose which one to use. If the drop-down below has no options to choose from, you can install the <a href="https://wordpress.org/plugins/geoip-detect/">GeoIP Detection plugin</a> to add the needed functionality.','xoogu-geotext');?></p>
                    </div>
                    <label for="geoFunc"><?php _e('Geo IP method:','xoogu-geotext');?></label>
                    <select id="geoFunc" name="options[geoFunc]">
                    <?php
					$geoFuncs = $this->detectGeoIP();
                    array_unshift($geoFuncs, '');
                    foreach ($geoFuncs as $key => $name) {
                        echo '<option value="'.$key.'"'.((!empty($formValues['geoFunc']) && $formValues['geoFunc']==$key) ? ' selected="selected"' : '').'>'.$name.'</option>';
                    }
                    ?></select>
                    <p><?php if (!empty($formValues['geoFunc'])) {
                        $sCountryCode = $this->getCountryCode();
                        if (!$sCountryCode) {
                            printf(__('Couldn\'t detect country details for your IP address %s', 'xoogu-geotext'), $_SERVER['REMOTE_ADDR']);
                        } else {
                            printf(__('Your country code is %s', 'xoogu-geotext'), $sCountryCode);
                        }
                        unset($info);
                    }?></p>
                </dd>
				<dt><?php _e('Groupings','xoogu-geotext');?></dt>
				<?php
					$i=0;
					foreach ( $formValues['groupings'] as $aGrouping) {
						echo '<dd><label for="groupingName-'.$i.'">Grouping name:</label><input type="text" id="grouping-'.$i.'" value="'.(!empty($aGrouping['name']) ? $aGrouping['name'] : '').'" name="groupings['.$i.'][name]" />
							<label for="groupingValue-'.$i.'">Countries:</label><input type="text" id="groupingValue-'.$i.'" value="'.(!empty($aGrouping['value']) ? $aGrouping['value'] : '').'" name="groupings['.$i.'][value]" />
						</dd>';
						$i++;
					}
				?>
            </dl>
        <?php
        //create the hidden nonce and referrer fields - https://codex.wordpress.org/Function_Reference/wp_nonce_field
        wp_nonce_field('save_settings', '_xoogu_geotext_nonce');
        submit_button();
        ?>
        </form>
		<script type="text/javascript">var urlToCountryList = '<?php echo plugins_url('country_select.html', __FILE__)."';</script>";
    }
	
	/**
	 *Gets the list of countries as a comma (and space) separated string for each grouping
	 *@return array The groupings and their country lists
	 */
	protected function getGroupingsCountryLists()
	{
		global $wpdb;
		$sql = "SELECT `grouping` AS name, GROUP_CONCAT(`country` SEPARATOR ', ') AS value FROM `".$wpdb->base_prefix.'xoogu_geoGroups` GROUP BY `grouping`';
		return $wpdb->get_results($sql, ARRAY_A);
	}
	
	 /**
     *Validate and save the POSTed settings when the settings form is submitted
     */
    public function validate_settings($input){
        
        //check nonce & referrer
        if(wp_verify_nonce($input['_xoogu_geotext_nonce'], 'save_settings') !== 1){
            wp_die(__('A valid nonce was not found.','xoogu-geotext') );
        }
        if(!current_user_can('manage_options')){
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="https://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','xoogu-geotext') );
        }
        $errors = array();
        
        //geoFunc
        if(!empty($input['options']['geoFunc'])){
            $geoFuncs = $this->detectGeoIP();
            if(empty($geoFuncs[$input['options']['geoFunc']])){
                $errors['geoFunc']= sprintf(__('%s is not a valid Geo IP method for this site','xoogu-geotext'), $input['options']['geoFunc']);
            }
            else{
                $this->_aOptions['geoFunc']=(string)$input['options']['geoFunc'];
            }
        }
        else{
            unset($this->_aOptions['geoFunc']);
        }
		
		//groupings
		$aAllCountries = array('af', 'ax', 'al', 'dz', 'as', 'ad', 'ao', 'ai', 'aq', 'ag', 'ar', 'am', 'aw', 'au', 'at', 'az', 'bs', 'bh', 'bd', 'bb', 'by', 'be', 'bz', 'bj', 'bm', 'bt', 'bo', 'bq', 'ba', 'bw', 'bv', 'br', 'io', 'bn', 'bg', 'bf', 'bi', 'cv', 'kh', 'cm', 'ca', 'ky', 'cf', 'td', 'cl', 'cn', 'cx', 'cc', 'co', 'km', 'cg', 'cd', 'ck', 'cr', 'ci', 'hr', 'cu', 'cw', 'cy', 'cz', 'dk', 'dj', 'dm', 'do', 'ec', 'eg', 'sv', 'gq', 'er', 'ee', 'et', 'fk', 'fo', 'fj', 'fi', 'fr', 'gf', 'pf', 'tf', 'ga', 'gm', 'ge', 'de', 'gh', 'gi', 'gr', 'gl', 'gd', 'gp', 'gu', 'gt', 'gg', 'gn', 'gw', 'gy', 'ht', 'hm', 'va', 'hn', 'hk', 'hu', 'is', 'in', 'id', 'ir', 'iq', 'ie', 'im', 'il', 'it', 'jm', 'jp', 'je', 'jo', 'kz', 'ke', 'ki', 'kp', 'kr', 'kw', 'kg', 'la', 'lv', 'lb', 'ls', 'lr', 'ly', 'li', 'lt', 'lu', 'mo', 'mk', 'mg', 'mw', 'my', 'mv', 'ml', 'mt', 'mh', 'mq', 'mr', 'mu', 'yt', 'mx', 'fm', 'md', 'mc', 'mn', 'me', 'ms', 'ma', 'mz', 'mm', 'na', 'nr', 'np', 'nl', 'nc', 'nz', 'ni', 'ne', 'ng', 'nu', 'nf', 'mp', 'no', 'om', 'pk', 'pw', 'ps', 'pa', 'pg', 'py', 'pe', 'ph', 'pn', 'pl', 'pt', 'pr', 'qa', 're', 'ro', 'ru', 'rw', 'bl', 'sh', 'kn', 'lc', 'mf', 'pm', 'vc', 'ws', 'sm', 'st', 'sa', 'sn', 'rs', 'sc', 'sl', 'sg', 'sx', 'sk', 'si', 'sb', 'so', 'za', 'gs', 'es', 'lk', 'sd', 'sr', 'sj', 'sz', 'se', 'ch', 'sy', 'tw', 'tj', 'tz', 'th', 'tl', 'tg', 'tk', 'to', 'tt', 'tn', 'tr', 'tm', 'tc', 'tv', 'ug', 'ua', 'ae', 'gb', 'us', 'um', 'uy', 'uz', 'vu', 've', 'vn', 'vg', 'vi', 'wf', 'eh', 'ye', 'zm', 'zw');
		$aGroupingsToAdd = array();
		foreach ($input['groupings'] as $aGrouping) {
			if (empty($aGrouping['name'])) {
				if (preg_match("/[A-Za-z]{2,2}/", $aGrouping['value'])) {
					$errors['groupings']['no_name'] = __('There is a grouping with countries selected for it, but no name has been chosen for the grouping','xoogu-geotext');
				}
				continue;
			}
			//check the country list contains at least two chars next to each other
			if (!preg_match("/[A-Za-z]{2,2}/", $aGrouping['value'])) {
				$errors['groupings'][$aGrouping['name']] = sprintf(__('Grouping %s does not have any countries listed to include in it','xoogu-geotext'), $aGrouping['name']);
				continue;
			}
			//add the grouping name to the list to add to the db
			$aGroupingsToAdd[$aGrouping['name']] = array();
			//split the list into country codes
			$aCountries = explode(',', strtolower(str_replace(' ', '', $aGrouping['value'])));
			//ensure no duplicates
			$aCountries = array_unique($aCountries);
			$aCountryNameErrors = array();
			//loop through the country codes for this grouping
			foreach ($aCountries as $sCountry) {
				//allow for people deleting country codes while leaving in commas
				if (empty($sCountry)) {
					continue;
				}
				//check country code is valid
				if (!in_array($sCountry, $aAllCountries)) {
					$aCountryNameErrors[] = sprintf(__('Grouping %s contains the country code %s, but this does not appear to be a valid country code','xoogu-geotext'), $aGrouping['name'], $sCountry);
					continue;
				}
				//add the country code to be added to the db for this grouping
				$aGroupingsToAdd[$aGrouping['name']][] = $sCountry;
			}
			//check our grouping did end up with some countries added to it
			if (!count($aGroupingsToAdd[$aGrouping['name']])) {
				$aCountryNameErrors[] = sprintf(__('Grouping %s does not have any valid countries listed to include in it','xoogu-geotext'), $aGrouping['name']);
				unset($aGroupingsToAdd[$aGrouping['name']]);
			}
			if (count($aCountryNameErrors)) {
				$errors['groupings'][$aGrouping['name']] = $aCountryNameErrors;
			}
		}
		
        if(!empty($errors)){
            return $errors;
        }
		//update plugin settings
        update_site_option('xoogu_geotext', $this->_aOptions);
		//Update groupings
		//Rather than trying to work out what records to add or delete, we just delete everything and add all afresh. Possibly more efficient?
		global $wpdb;
		$sSQL = 'TRUNCATE TABLE `'.$wpdb->base_prefix.'xoogu_geoGroups`';
		$wpdb->query($sSQL);
		if (!empty($aGroupingsToAdd)) {
			$aSQL = array();
			foreach ($aGroupingsToAdd as $name => $aValues) {
				$aSQL[] = "('".implode("', '".$name."'), ('", $aValues)."', '".$name."')";
			}
			$sSQL = 'INSERT INTO `'.$wpdb->base_prefix.'xoogu_geoGroups` VALUES '.implode(', ', $aSQL);
			$wpdb->query($sSQL);
		}
    }
	
	
	/**
	 *Prints out a list of errors based on the given array of error messages
	 *@param array $aErrors The array of error messages
	 */
	protected function printErrors($aErrors)
	{
		echo '<ul>';
		foreach ($aErrors as $msg) {
			if (is_array($msg)) {
				echo '<li>'; $this->printErrors($msg); echo '</li>';
			} else {
				echo '<li>'.$msg.'</li>';
			}
		}
		echo '</ul>';
	}
	
    
    /**
     *Filter callback attached to plugin_action_links_(plugin filename) hook
     *Adds a settings link to the plugin in the plugins list
     *@param array $links The links to print for this plugin entry
     *@return array The links to print for this plugin entry
     */
    public function plugin_action_links($links){
        array_unshift($links, '<a href="'. get_admin_url(null, 'options-general.php?page=xoogu_geotext') .'">'.__('Settings','xoogu-geotext').'</a>');
        return $links;
    }
	
    
    /**
     *Filter callback attached to plugin_action_links_(plugin filename) hook
     *Adds a settings link to the plugin in the plugins list
     *@param array $links The links to print for this plugin entry
     *@return array The links to print for this plugin entry
     */
    public function plugin_action_links_network($links){
        array_unshift($links, '<a href="'. network_admin_url('settings.php?page=xoogu_geotext') .'">'.__('Settings','xoogu-geotext').'</a>');
        return $links;
    }
	
	
    /**
     *Creates the options entries in the db
     */
    public function activate($network)
	{
        //check the current user has permission to activate the plugin
        if (!current_user_can('activate_plugins')) {
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-geotext' ) );
        }
        
        //check the nonce is valid (will automatically die if it is not)
        $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
        check_admin_referer("activate-plugin_{$plugin}");
        
        global $wpdb;
        $wpdb->show_errors();
		
		if (empty($this->_aOptions)) {
            if ( ($aGeoFuncs=$this->detectGeoIP()) ) {
                $this->_aOptions = array('geoFunc' => key($aGeoFuncs));
            }
			add_site_option('xoogu_geotext', $this->_aOptions);
        }
		
		
		//Countries to Groups map table
        $sTblGroups = $wpdb->base_prefix.'xoogu_geoGroups';
		if ($wpdb->get_var("SHOW TABLES LIKE '$sTblGroups'") != $sTblGroups) {
            //create the table
            $sql = 'CREATE TABLE '.$sTblGroups.' (
                    country char(2) NOT NULL,
					grouping varchar(55) NOT NULL,
                    PRIMARY KEY  (country,grouping)
            ) '.$wpdb->get_charset_collate();
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
            //If the table wasn't created then die
            if ($wpdb->get_var("SHOW TABLES LIKE '$sTblGroups'") != $sTblGroups) {
                //couldn't create table so die
                wp_die(new WP_Error('tblCreation', sprintf(__('Creation of table %s failed.','xoogu-geotext'), $sTblGroups), $sql));
            }
            //Fill the table
            $sql= "INSERT INTO `$sTblGroups` VALUES ";
            if (($handle = fopen(dirname(__FILE__).'/countries_groupings_map.csv', 'r')) !== FALSE) {
                fgetcsv($handle); //don't need first row
				$aData = fgetcsv($handle);
				$sql.= ' ("'.$aData[0].'", "'.$aData[1].'")';
                while (($aData = fgetcsv($handle)) !== FALSE) {
                    $sql.= ', ("'.$aData[0].'", "'.$aData[1].'")';
                }
                fclose($handle);
            } else {
                //couldn't open data file so die
                wp_die(new WP_Error('tblCreation', sprintf(__('Couldn\'t open data file %s.','xoogu-geotext'), 'countries_groupings_map.csv') ));
            }
            //var_dump($sql);
            if (!$wpdb->query($sql)) {
                //couldn't fill table so die
                wp_die(new WP_Error('tblCreation', sprintf(__('Couldn\'t insert data into %s.','xoogu-geotext'),$sTblGroups ),$sql ));
			}
        }
		
		update_site_option('xoogu_geotext_version', self::$sVersion);
    }
    
	
    /**
     *Remove database records when plugin is removed
     */
    public static function uninstall(){
        
        //check permissions
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-geotext' ) );
        }
        //check nonce
        check_admin_referer('bulk-plugins');
        
        //multisite network uninstall (can only ever be a network uninstall in multisite)
        if(is_multisite()){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to delete the plugin', 'xoogu-geotext' ) );
            }
        }
        
        //remove the db table
        global $wpdb;
        $wpdb->query('DROP TABLE '.$wpdb->base_prefix.'xoogu_geoGroups');
        
        //remove the options from the db
        delete_site_option('xoogu_geotext');
        delete_site_option('xoogu_geotext_version');
    }
	
	
    /**
     *Detects installed methods of getting the country code of a visitor
     *@return array List of supported methods
     */
    public function detectGeoIP()
	{
        //cache result - switch_blog for WPMU does not modify plugins loaded so this func will always give the same result for each blog
        static $aGeoFuncs;
        if (is_array($aGeoFuncs)) {
            return $aGeoFuncs;
        }
        global $wpdb;
        $aGeoFuncs=array();
        if (isset($_SERVER['GEOIP_COUNTRY_CODE'])) {
            $aGeoFuncs['server'] = __('Server has GeoIP extension installed','xoogu-geotext');
        }
        //PHP GeoIP extension - http://www.php.net/manual/en/function.geoip-country-code-by-name.php
        if (extension_loaded('geoip')) {
            $aGeoFuncs['php'] = __('PHP GeoIP extension','xoogu-geotext');
        }
        //GeoIP Detection plugin - https://wordpress.org/plugins/geoip-detect/
        if (is_plugin_active('geoip-detect/geoip-detect.php') && function_exists('geoip_detect2_get_info_from_current_ip')) {
			$aGeoFuncs['geoip-detect2'] = __('GeoIP Detection plugin v2','xoogu-geotext');
        }
        //Try ip2nation table - https://wordpress.org/plugins/wp-ip2nation-installer/
        if ($wpdb->get_var("SHOW TABLES LIKE 'ip2nation'")) {
            $aGeoFuncs['ip2nation'] = __('ip2nation database table','xoogu-geotext');
        }
        //move the pointer to first item
        reset($aGeoFuncs);
        return $aGeoFuncs;
    }
}