<?php
class Xoogu_GeoText_Admin_MCE{
	
	//Should really get the version from the main plugin file, but pointless to load that whole script just for a version no.
    public static $sVersion = '2016-04-26';
	
    /**
     *Initialise the plugin
     */
    public function __construct(){
		//tiny mce (call directly rather than hooking into admin_init as we are being loaded from the current_screen hook, which is called after the admin_init hook)
		$this->mce_init();
		//quicktags and shared
		add_action('admin_enqueue_scripts', array($this, 'loadAdminjscss'));
        add_action('wp_ajax_xgtsh_mce_popup', array($this, 'mce_popup'));
		add_action('wp_ajax_xgtsh_quicktags_dialog', array($this, 'quicktags_dialog'));
    }
    
	
	/**
	 *Loads the scripts and CSS needed to add the shortcode button and settings dialog for the tinyMCE and quicktags editors
	 */
	public function loadAdminjscss(){
		//quicktags
		//wordpress does include some jquery ui dialog css, but it is rather minimal and results in a transparent div
		//wp_enqueue_style('wp-jquery-ui-dialog');
		wp_enqueue_style('jquery-ui', '//code.jquery.com/ui/1.10.3/themes/smoothness/jquery-ui.css');
		wp_enqueue_script('xoogu_geotext_quicktags', plugins_url( 'quicktags.js' , __FILE__ ), array('jquery-ui-dialog', 'jquery','quicktags'), self::$sVersion, true );
		wp_enqueue_style( 'xoogu_geotext_mce', plugins_url( 'mce_quicktags_shared.css' , __FILE__ ), null, self::$sVersion );
		
		//shared
		wp_enqueue_script('xoogu_geotext_mce_quicktags_shared', plugins_url( 'mce_quicktags_shared.js' , __FILE__ ), array('jquery'), self::$sVersion, true );
		wp_localize_script( 'xoogu_geotext_mce_quicktags_shared', 'xgtL10n', array(
			'show_description' => __('Show description', 'xoogu-geotext'),
			'hide_description' => __('Hide description', 'xoogu-geotext'),
			'help_for_setting' => __('Help for this setting', 'xoogu-geotext'),
		    'geoTextbutton_title' => __('Geo localize this text', 'xoogu-geotext'),
		    'geoContentbutton_title' => __('Geo localize this content', 'xoogu-geotext'),
		    'addCountry' => __('Add another country', 'xoogu-geotext'),
		    'addGrouping' => __('Add another grouping', 'xoogu-geotext'),
			'mce_plugin_longname' => sprintf(__('%s %s shortcode settings', 'xoogu-geotext'), 'Xoogu', __('GeoText', 'xoogu-geotext')),
			'dialogGeoTextTitle'  => sprintf(__('%s %s shortcode settings', 'xoogu-geotext'), 'Xoogu', __('GeoText', 'xoogu-geotext')),
			'dialogGeoContentTitle'  => sprintf(__('%s %s shortcode settings', 'xoogu-geotext'), 'Xoogu', __('GeoContent', 'xoogu-geotext'))
		) );
	}
    
	
    /**
     *Adds the filters needed to add the button to the TinyMCE Editor
     */
    public function mce_init() {
        add_filter( 'mce_external_plugins', array($this, 'mce_add_buttons') );
        add_filter( 'mce_buttons', array($this,'mce_register_buttons') );
    }
    
	
    /**
     *Adds the TinyMCE plugin javascript
     *@param Array $plugin_array Array of TinyMCE plugins to be loaded
     *@return Array Array of TinyMCE plugins to be loaded
     */
    public function mce_add_buttons( $plugin_array ) {
        $plugin_array['xoogu_geotext'] = plugins_url( 'mce-button.js' , __FILE__ );
        return $plugin_array;
    }
    
	
    /**
     *Registers the button to display in the TinyMCE Editor
     *@param Array $plugin_array Array of TinyMCE buttons to be available
     *@return Array Array of TinyMCE buttons to be available
     */
    function mce_register_buttons( $buttons ) {
        array_push( $buttons, 'xoogu_geoText' );
        array_push( $buttons, 'xoogu_geoContent' );
        return $buttons;
    }
    
	/**
     *Display the settings page
     */
    public function display_mce_form_geoContent()
	{
		
		//Exclude
		//select country
		//repeat as necessary
		
		//Countries
		//select country
		//repeat as necessary
		
		//Groupings
		//select grouping
		//repeat as necessary
		
		?>
        <form autocomplete="off" id="xgtsh_form" >
            <ul class="geoContent">
				<li>
					<h3><?php _e('Choose countries that should see this content','xoogu-geotext');?></h3>
					<div id="xgtsh_country_0_container"><label for="xgtsh_country_0"><?php _e('Choose a country:','xoogu-geotext');?></label>
					<select name="xgtsh[country][0]" id="xgtsh_country_0">
						<option></option>
						<?php $countryListFile=dirname(__FILE__).'/'.__('country_select.html','xoogu-geotext');
						if (file_exists($countryListFile)) {
						  echo file_get_contents($countryListFile);
						} ?>
					</select></div>
				</li>
				<li>
					<h3><?php _e('Choose grouping that should see this content','xoogu-geotext');?></h3>
					<div id="xgtsh_grouping_0_container"><label for="xgtsh_grouping_0"><?php _e('Choose a grouping:','xoogu-geotext');?></label>
					<select name="xgtsh[grouping][0]" id="xgtsh_grouping_0">
						<option></option>
						<?php $aGroupings = $this->getGroupingsList();
						foreach ($aGroupings as $aGrouping) {
							echo "<option>$aGrouping</option>";
						} ?>
					</select></div>
				</li>
				<li>
					<h3><?php _e('Choose countries to exclude from seeing this content','xoogu-geotext');?></h3>
					<div id="xgtsh_excludeCountry_0_container"><label for="xgtsh_excludeCountry_0"><?php _e('Choose a country:','xoogu-geotext');?></label>
					<select name="xgtsh[excludeCountry][0]" id="xgtsh_excludeCountry_0">
						<option></option>
						<?php 
						if (file_exists($countryListFile)) {
						  echo file_get_contents($countryListFile);
						} ?>
					</select></div>
				</li>
				<li>
					<h3><?php _e('Choose groupings to exclude from seeing this content','xoogu-geotext');?></h3>
					<div id="xgtsh_excludeGrouping_0_container"><label for="xgtsh_excludeGrouping_0"><?php _e('Choose a grouping:','xoogu-geotext');?></label>
					<select name="xgtsh[excludeGrouping][0]" id="xgtsh_excludeGrouping_0">
						<option></option>
						<?php
						foreach ($aGroupings as $aGrouping) {
							echo "<option>$aGrouping</option>";
						} ?>
					</select></div>
				</li>
				<li>
					<h3><?php _e('The content enclosed by the shortcode','xoogu-geotext');?></h3>
					<textarea id="xgtsh_content"></textarea>
				</li>
			</ul>
		</form><?php
	}
	
	public function getGroupingsList()
	{
		static $aGroupings;
		if ($aGroupings) {
			return $aGroupings;
		}
		global $wpdb;
		$aGroupings = $wpdb->get_results('SELECT DISTINCT `grouping` FROM '.$wpdb->base_prefix.'xoogu_geoGroups', ARRAY_N);
		//convert the results so that each 'row' is just the fetched value rather than an array containing the fetched value
		foreach ($aGroupings as &$grouping) {
			$grouping = $grouping[0];
		}
		unset($grouping);
		return $aGroupings;
	}
	
    /**
     *Display the settings page
     */
    public function display_mce_form_geoText(){
		
		//select country or grouping
		//type text
		//repeat as necessary
		
		?>
        <form autocomplete="off" id="xgtsh_form" >
            <ul>
				<li>
					<ul id="xgtsh_0">
						<li>
							<div class="desc">
								<p><?php _e('Choose a country or grouping to display alternate text to','xoogu-geotext');?></p>
							</div>
							<p><label for="xgtsh_0_country"><?php _e('Choose a country:','xoogu-geotext');?></label>
							<select name="xgtsh[0][country]" id="xgtsh_0_country">
								<option></option>
								<?php $countryListFile=dirname(__FILE__).'/'.__('country_select.html','xoogu-geotext');
								if (file_exists($countryListFile)) {
								  echo file_get_contents($countryListFile);
								} ?>
							</select></p>
							or
							<p><label for="xgtsh_0_grouping"><?php _e('Choose a grouping:','xoogu-geotext');?></label>
							<select name="xgtsh[0][grouping]" id="xgtsh_0_grouping">
								<option></option>
								<?php $aGroupings = $this->getGroupingsList();
								foreach ($aGroupings as $aGrouping) {
									echo "<option>$aGrouping</option>";
								} ?>
							</select></p>
						</li>
						<li>
							<div class="desc">
								<p><?php _e('The text that will be shown if the user\'s country matches this country / grouping.','xoogu-geotext');?></p>
							</div>
							<label for="xgtsh_0_text"><?php _e('Text to display for this country / grouping:','xoogu-geotext');?></label>
							<input type="text" value="" name="xgtsh[0]text" id="xgtsh_0_text" />
						</li>
					</ul>
				</li>
				<li>
					<div class="desc">
						<p><?php _e('The text that will be wrapped in the shortcode, i.e. the default text shown if none of the countries or groupings match.','xoogu-geotext');?></p>
					</div>
					<label for="xgtsh_content"><?php _e('Default text:','xoogu-geotext');?></label>
                    <input type="text" value="" name="xgtsh_content" id="xgtsh_content" />
				</li>
            </ul>
        </form><?php
    }
	
	
	/**
	 *Outputs the settings form the quicktags dialog AJAX call
	 */
	public function quicktags_dialog(){
		header('Content-type: text/html');
		//cache the page
		//header('Expires: '.gmdate('D, d M Y H:i:s \G\M\T', time() + 86400));
		header('Pragma: cache');
		header('Cache-Control: public, max-age=86400');
		if ($_GET['type'] == 'geoText') {
			$this->display_mce_form_geoText();
		} elseif ($_GET['type'] == 'geoContent') {
			$this->display_mce_form_geoContent();
		}
		exit();
	}
	
	
	/**
	 *Outputs an HTML page with the shortcode settings, called by the tinyMCE shortcode button
	 */
	public function mce_popup(){
		header('Content-type: text/html');
		//cache the page
		//header('Expires: '.gmdate('D, d M Y H:i:s \G\M\T', time() + 86400));
		header('Pragma: cache');
		header('Cache-Control: public, max-age=86400');
		wp_enqueue_script('jquery');
		//load the tiny MCE popup js (included with wordpress)
		//wp_enqueue_script('wpdialogs-popup'); //this doesn't work, seems not to just be a minified version of below
		wp_enqueue_script('tiny_mce_popup', get_site_url(null, '/wp-includes/js/tinymce/tiny_mce_popup.js'));
		//load our script for handling the settings form
		wp_enqueue_script('xgtsh_mce_popup', plugins_url( 'mce_popup.js' , __FILE__ ), array('jquery'), self::$sVersion);
		wp_localize_script( 'xgtsh_mce_popup', 'xgtL10n', array(
			'cancel' => __('Cancel', 'xoogu-geotext'),
			'insert' => __('Insert shortcode', 'xoogu-geotext')
		) );
		//load the style for the settings form
		wp_enqueue_style( 'xoogu_geotext_mce', plugins_url( 'mce_quicktags_shared.css' , __FILE__ ), null, self::$sVersion );
		
		
		?><!DOCTYPE html>
		<html>
		<head>
			<title><?php printf(__('%s %s shortcode settings', 'xoogu-geotext'), 'Xoogu', $_GET['type'] == 'geoText' ? __('GeoText', 'xoogu-geotext') : __('GeoContent', 'xoogu-geotext'));?></title>
			<?php wp_head();?>
		</head>
		
		<body>
		<?php
		if ($_GET['type'] == 'geoText') {
			$this->display_mce_form_geoText();
		} elseif ($_GET['type'] == 'geoContent') {
			$this->display_mce_form_geoContent();
		} ?>
		<?php wp_footer();?>
		</body>
		</html>
		<?php exit();
	}
}